"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var _react = _interopRequireWildcard(require("react"));
var _propTypes = _interopRequireDefault(require("prop-types"));
var _lodash = _interopRequireDefault(require("lodash.debounce"));
var _autoprefix = _interopRequireDefault(require("./autoprefix"));
function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }
function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }
function autoprefixes(styles) {
  return Object.keys(styles).reduce((obj, key) => (obj[key] = (0, _autoprefix.default)(styles[key]), obj), {});
}
const styles = autoprefixes({
  wrapper: {
    position: 'fixed',
    width: 0,
    height: 0,
    top: 0,
    left: 0
  },
  dim: {
    position: 'fixed',
    left: 0,
    right: 0,
    top: 0,
    bottom: 0,
    zIndex: 0,
    background: 'rgba(0, 0, 0, 0.2)',
    opacity: 1
  },
  dimAppear: {
    opacity: 0
  },
  dimTransparent: {
    pointerEvents: 'none'
  },
  dimHidden: {
    opacity: 0
  },
  dock: {
    position: 'fixed',
    zIndex: 1,
    boxShadow: '0 0 4px rgba(0, 0, 0, 0.3)',
    background: 'white',
    left: 0,
    top: 0,
    width: '100%',
    height: '100%'
  },
  dockHidden: {
    opacity: 0
  },
  dockResizing: {
    transition: 'none'
  },
  dockContent: {
    width: '100%',
    height: '100%',
    overflow: 'auto'
  },
  resizer: {
    position: 'absolute',
    zIndex: 2,
    opacity: 0
  }
});
function getTransitions(duration) {
  return ['left', 'top', 'width', 'height'].map(p => `${p} ${duration / 1000}s ease-out`);
}
function getDockStyles(_ref, _ref2) {
  let {
    fluid,
    dockStyle,
    dockHiddenStyle,
    duration,
    position,
    isVisible
  } = _ref;
  let {
    size,
    isResizing,
    fullWidth,
    fullHeight
  } = _ref2;
  let posStyle;
  const absSize = fluid ? `${size * 100}%` : `${size}px`;
  function getRestSize(fullSize) {
    return fluid ? `${100 - size * 100}%` : `${fullSize - size}px`;
  }
  switch (position) {
    case 'left':
      posStyle = {
        width: absSize,
        left: isVisible ? 0 : '-' + absSize
      };
      break;
    case 'right':
      posStyle = {
        left: isVisible ? getRestSize(fullWidth) : fullWidth,
        width: absSize
      };
      break;
    case 'top':
      posStyle = {
        top: isVisible ? 0 : '-' + absSize,
        height: absSize
      };
      break;
    case 'bottom':
      posStyle = {
        top: isVisible ? getRestSize(fullHeight) : fullHeight,
        height: absSize
      };
      break;
  }
  const transitions = getTransitions(duration);
  return [styles.dock, (0, _autoprefix.default)({
    transition: [...transitions, !isVisible && `opacity 0.01s linear ${duration / 1000}s`].filter(t => t).join(',')
  }), dockStyle, (0, _autoprefix.default)(posStyle), isResizing && styles.dockResizing, !isVisible && styles.dockHidden, !isVisible && dockHiddenStyle];
}
function getDimStyles(_ref3, _ref4) {
  let {
    dimMode,
    dimStyle,
    duration,
    isVisible
  } = _ref3;
  let {
    isTransitionStarted
  } = _ref4;
  return [styles.dim, (0, _autoprefix.default)({
    transition: `opacity ${duration / 1000}s ease-out`
  }), dimStyle, dimMode === 'transparent' && styles.dimTransparent, !isVisible && styles.dimHidden, isTransitionStarted && isVisible && styles.dimAppear, isTransitionStarted && !isVisible && styles.dimDisappear];
}
function getResizerStyles(position) {
  let resizerStyle;
  const size = 10;
  switch (position) {
    case 'left':
      resizerStyle = {
        right: -size / 2,
        width: size,
        top: 0,
        height: '100%',
        cursor: 'col-resize'
      };
      break;
    case 'right':
      resizerStyle = {
        left: -size / 2,
        width: size,
        top: 0,
        height: '100%',
        cursor: 'col-resize'
      };
      break;
    case 'top':
      resizerStyle = {
        bottom: -size / 2,
        height: size,
        left: 0,
        width: '100%',
        cursor: 'row-resize'
      };
      break;
    case 'bottom':
      resizerStyle = {
        top: -size / 2,
        height: size,
        left: 0,
        width: '100%',
        cursor: 'row-resize'
      };
      break;
  }
  return [styles.resizer, (0, _autoprefix.default)(resizerStyle)];
}
function getFullSize(position, fullWidth, fullHeight) {
  return position === 'left' || position === 'right' ? fullWidth : fullHeight;
}
class Dock extends _react.Component {
  state = {
    isControlled: typeof this.props.size !== 'undefined',
    size: this.props.size || this.props.defaultSize,
    isDimHidden: !this.props.isVisible,
    fullWidth: window.innerWidth,
    fullHeight: window.innerHeight,
    isTransitionStarted: false,
    isWindowResizing: false
  };
  static propTypes = {
    position: _propTypes.default.oneOf(['left', 'right', 'top', 'bottom']),
    zIndex: _propTypes.default.number,
    fluid: _propTypes.default.bool,
    size: _propTypes.default.number,
    defaultSize: _propTypes.default.number,
    dimMode: _propTypes.default.oneOf(['none', 'transparent', 'opaque']),
    isVisible: _propTypes.default.bool,
    onVisibleChange: _propTypes.default.func,
    onSizeChange: _propTypes.default.func,
    dimStyle: _propTypes.default.object,
    dockStyle: _propTypes.default.object,
    duration: _propTypes.default.number
  };
  static defaultProps = {
    position: 'left',
    zIndex: 99999999,
    fluid: true,
    defaultSize: 0.3,
    dimMode: 'opaque',
    duration: 200
  };
  componentDidMount() {
    window.addEventListener('touchend', this.handleMouseUp);
    window.addEventListener('mouseup', this.handleMouseUp);
    window.addEventListener('touchmove', this.handleMouseMove);
    window.addEventListener('mousemove', this.handleMouseMove);
    window.addEventListener('resize', this.handleResize);
    this.updateWindowSize();
  }
  componentWillUnmount() {
    window.removeEventListener('touchend', this.handleMouseUp);
    window.removeEventListener('mouseup', this.handleMouseUp);
    window.removeEventListener('touchmove', this.handleMouseMove);
    window.removeEventListener('mousemove', this.handleMouseMove);
    window.removeEventListener('resize', this.handleResize);
  }
  UNSAFE_componentWillReceiveProps(nextProps) {
    const isControlled = typeof nextProps.size !== 'undefined';
    this.setState({
      isControlled
    });
    if (isControlled && nextProps.size && this.props.size !== nextProps.size) {
      this.setState({
        size: nextProps.size
      });
    } else if (this.props.fluid !== nextProps.fluid) {
      this.updateSize(nextProps);
    }
    if (this.props.isVisible !== nextProps.isVisible) {
      this.setState({
        isTransitionStarted: true
      });
    }
  }
  updateSize(props) {
    const {
      fullWidth,
      fullHeight
    } = this.state;
    this.setState({
      size: props.fluid ? this.state.size / getFullSize(props.position, fullWidth, fullHeight) : getFullSize(props.position, fullWidth, fullHeight) * this.state.size
    });
  }
  componentDidUpdate(prevProps) {
    if (this.props.isVisible !== prevProps.isVisible) {
      if (!this.props.isVisible) {
        window.setTimeout(() => this.hideDim(), this.props.duration);
      } else {
        this.setState({
          isDimHidden: false
        });
      }
      window.setTimeout(() => this.setState({
        isTransitionStarted: false
      }), 0);
    }
  }
  transitionEnd = () => {
    this.setState({
      isTransitionStarted: false
    });
  };
  hideDim = () => {
    if (!this.props.isVisible) {
      this.setState({
        isDimHidden: true
      });
    }
  };
  render() {
    const {
      children,
      zIndex,
      dimMode,
      position,
      isVisible
    } = this.props;
    const {
      isResizing,
      size,
      isDimHidden
    } = this.state;
    const dimStyles = Object.assign({}, ...getDimStyles(this.props, this.state));
    const dockStyles = Object.assign({}, ...getDockStyles(this.props, this.state));
    const resizerStyles = Object.assign({}, ...getResizerStyles(position));
    return /*#__PURE__*/_react.default.createElement("div", {
      style: Object.assign({}, styles.wrapper, {
        zIndex
      })
    }, dimMode !== 'none' && !isDimHidden && /*#__PURE__*/_react.default.createElement("div", {
      style: dimStyles,
      onClick: this.handleDimClick
    }), /*#__PURE__*/_react.default.createElement("div", {
      style: dockStyles
    }, /*#__PURE__*/_react.default.createElement("div", {
      style: resizerStyles,
      onMouseDown: this.handleMouseDown,
      onTouchStart: this.handleMouseDown
    }), /*#__PURE__*/_react.default.createElement("div", {
      style: styles.dockContent
    }, typeof children === 'function' ? children({
      position,
      isResizing,
      size,
      isVisible
    }) : children)));
  }
  handleDimClick = () => {
    if (this.props.dimMode === 'opaque') {
      this.props.onVisibleChange && this.props.onVisibleChange(false);
    }
  };
  handleResize = () => {
    if (window.requestAnimationFrame) {
      window.requestAnimationFrame(this.updateWindowSize.bind(this, true));
    } else {
      this.updateWindowSize(true);
    }
  };
  updateWindowSize = windowResize => {
    const sizeState = {
      fullWidth: window.innerWidth,
      fullHeight: window.innerHeight
    };
    if (windowResize) {
      this.setState({
        ...sizeState,
        isResizing: true,
        isWindowResizing: windowResize
      });
      this.debouncedUpdateWindowSizeEnd();
    } else {
      this.setState(sizeState);
    }
  };
  updateWindowSizeEnd = () => {
    this.setState({
      isResizing: false,
      isWindowResizing: false
    });
  };
  debouncedUpdateWindowSizeEnd = (0, _lodash.default)(this.updateWindowSizeEnd, 30);
  handleWrapperLeave = () => {
    this.setState({
      isResizing: false
    });
  };
  handleMouseDown = () => {
    this.setState({
      isResizing: true
    });
  };
  handleMouseUp = () => {
    this.setState({
      isResizing: false
    });
  };
  handleMouseMove = e => {
    if (!this.state.isResizing || this.state.isWindowResizing) return;
    if (!e.touches) e.preventDefault();
    const {
      position,
      fluid
    } = this.props;
    const {
      fullWidth,
      fullHeight,
      isControlled
    } = this.state;
    let {
      clientX: x,
      clientY: y
    } = e;
    if (e.touches) {
      x = e.touches[0].clientX;
      y = e.touches[0].clientY;
    }
    let size;
    switch (position) {
      case 'left':
        size = fluid ? x / fullWidth : x;
        break;
      case 'right':
        size = fluid ? (fullWidth - x) / fullWidth : fullWidth - x;
        break;
      case 'top':
        size = fluid ? y / fullHeight : y;
        break;
      case 'bottom':
        size = fluid ? (fullHeight - y) / fullHeight : fullHeight - y;
        break;
    }
    this.props.onSizeChange && this.props.onSizeChange(size);
    if (!isControlled) {
      this.setState({
        size
      });
    }
  };
}
exports.default = Dock;